package gg.malloc.defense;

import gg.malloc.defense.engine.GameRunner;

import java.util.Collection;
import java.util.HashSet;
import java.util.ArrayList;

import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityCombustEvent;
import org.bukkit.event.entity.EntityTargetEvent;
import org.bukkit.event.entity.FoodLevelChangeEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.world.LootGenerateEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.entity.Player;

public class GameEventHandler implements Listener {
  Collection<GameRunner> m_runners;

  public GameEventHandler() {
    m_runners = new HashSet<GameRunner>();
  }

  public void addRunner(GameRunner runner) {
    m_runners.add(runner);
  }

  public void clear() {
    m_runners.clear();
  }

  @EventHandler
  public void onEntityDeath(EntityDeathEvent evt) {
    for(GameRunner r : m_runners) {
      r.handleEntityDeath(evt.getEntity());
    }
  }

  @EventHandler
  public void onPlayerQuit(PlayerQuitEvent evt) {
    for(GameRunner r : m_runners) {
      r.removePlayer(evt.getPlayer());
    }
  }

  @EventHandler
  public void onEntityCombust(EntityCombustEvent evt) {
    evt.setCancelled(true);
  }

  @EventHandler
  public void onEntityTarget(EntityTargetEvent evt) {
    for(GameRunner r : m_runners) {
      r.handleEntityRetargeting(evt);
    }
  }

  @EventHandler
  public void onEntityDamage(EntityDamageEvent evt) {
    for(GameRunner r : m_runners) {
      r.handleEntityDamage(evt);
    }
  }

  @EventHandler
  public void onLootGenerated(LootGenerateEvent evt) {
    if (evt.getEntity() != null) {
      ArrayList<ItemStack> emptyLoot = new ArrayList<>();
      evt.setLoot(emptyLoot);
    }
  }

  @EventHandler
  public void onFoodLevelChange(FoodLevelChangeEvent evt) {
    evt.setCancelled(true);
  }
}
